@@ -39,7 +39,7 @@
 
 #include "common/native_helper.h"
 #ifdef HAVE_WAYLAND_BACKEND
-#include "common/native_wayland_drm_bufmgr_helper.h"
+#include "common/native_wayland_drm_bufmgr.h"
 #endif
 
 #ifdef GLX_DIRECT_RENDERING
@@ -60,9 +60,6 @@ struct dri2_display {
    int num_configs;
 
    struct util_hash_table *surfaces;
-#ifdef HAVE_WAYLAND_BACKEND
-   struct wl_drm *wl_server_drm; /* for EGL_WL_bind_wayland_display */
-#endif
 };
 
 struct dri2_surface {
@@ -293,6 +290,12 @@ dri2_surface_flush_frontbuffer(struct native_surface *nsurf)
 {
    struct dri2_surface *dri2surf = dri2_surface(nsurf);
    struct dri2_display *dri2dpy = dri2surf->dri2dpy;
+   struct native_display *ndpy = &dri2dpy->base;
+   struct pipe_context *pipe = ndpy_get_copy_context(ndpy);
+
+   /* flush buffer */
+   pipe->flush_resource(pipe, dri2surf->textures[NATIVE_ATTACHMENT_FRONT_LEFT]);
+   pipe->flush(pipe, NULL, 0);
 
    /* copy to real front buffer */
    if (dri2surf->have_fake)
@@ -316,9 +319,14 @@ dri2_surface_swap_buffers(struct native_surface *nsurf, int num_rects,
 {
    struct dri2_surface *dri2surf = dri2_surface(nsurf);
    struct dri2_display *dri2dpy = dri2surf->dri2dpy;
+   struct native_display *ndpy = &dri2dpy->base;
+   struct pipe_context *pipe = ndpy_get_copy_context(ndpy);
 
    /* copy to front buffer */
    if (dri2surf->have_back) {
+      pipe->flush_resource(pipe, dri2surf->textures[NATIVE_ATTACHMENT_BACK_LEFT]);
+      pipe->flush(pipe, NULL, 0);
+
       if (num_rects > 0)
          x11_drawable_copy_buffers_region(dri2dpy->xscr, dri2surf->drawable,
                num_rects, rects,
@@ -757,6 +765,10 @@ dri2_display_destroy(struct native_display *ndpy)
    if (dri2dpy->surfaces)
       util_hash_table_destroy(dri2dpy->surfaces);
 
+#ifdef HAVE_WAYLAND_BACKEND
+   wayland_drm_bufmgr_destroy(ndpy->wayland_bufmgr);
+#endif
+
    if (dri2dpy->xscr)
       x11_screen_destroy(dri2dpy->xscr);
    if (dri2dpy->own_dpy)
@@ -785,6 +797,19 @@ dri2_display_invalidate_buffers(struct x11_screen *xscr, Drawable drawable,
          &dri2surf->base, dri2surf->server_stamp);
 }
 
+#ifdef HAVE_WAYLAND_BACKEND
+
+static int
+dri2_display_authenticate(void *user_data, uint32_t magic)
+{
+   struct native_display *ndpy = user_data;
+   struct dri2_display *dri2dpy = dri2_display(ndpy);
+
+   return x11_screen_authenticate(dri2dpy->xscr, magic);
+}
+
+#endif /* HAVE_WAYLAND_BACKEND */
+
 /**
  * Initialize DRI2 and pipe screen.
  */
@@ -816,6 +841,13 @@ dri2_display_init_screen(struct native_display *ndpy)
       return FALSE;
    }
 
+#ifdef HAVE_WAYLAND_BACKEND
+   dri2dpy->base.wayland_bufmgr = wayland_drm_bufmgr_create(
+         dri2_display_authenticate, dri2dpy,
+         x11_screen_get_device_name(dri2dpy->xscr));
+
+#endif
+
    return TRUE;
 }
 
@@ -832,66 +864,6 @@ dri2_display_hash_table_compare(void *key1, void *key2)
    return ((char *) key1 - (char *) key2);
 }
 
-#ifdef HAVE_WAYLAND_BACKEND
-
-static int
-dri2_display_authenticate(void *user_data, uint32_t magic)
-{
-   struct native_display *ndpy = user_data;
-   struct dri2_display *dri2dpy = dri2_display(ndpy);
-
-   return x11_screen_authenticate(dri2dpy->xscr, magic);
-}
-
-static struct wayland_drm_callbacks wl_drm_callbacks = {
-   dri2_display_authenticate,
-   egl_g3d_wl_drm_helper_reference_buffer,
-   egl_g3d_wl_drm_helper_unreference_buffer
-};
-
-static boolean
-dri2_display_bind_wayland_display(struct native_display *ndpy,
-                                  struct wl_display *wl_dpy)
-{
-   struct dri2_display *dri2dpy = dri2_display(ndpy);
-
-   if (dri2dpy->wl_server_drm)
-      return FALSE;
-
-   ndpy->wl_server_drm = wayland_drm_init(wl_dpy,
-         x11_screen_get_device_name(dri2dpy->xscr),
-         &wl_drm_callbacks, ndpy, 0);
-
-   if (!dri2dpy->wl_server_drm)
-      return FALSE;
-   
-   return TRUE;
-}
-
-static boolean
-dri2_display_unbind_wayland_display(struct native_display *ndpy,
-                                    struct wl_display *wl_dpy)
-{
-   struct dri2_display *dri2dpy = dri2_display(ndpy);
-
-   if (!dri2dpy->wl_server_drm)
-      return FALSE;
-
-   wayland_drm_uninit(dri2dpy->wl_server_drm);
-   dri2dpy->wl_server_drm = NULL;
-
-   return TRUE;
-}
-
-static struct native_display_wayland_bufmgr dri2_display_wayland_bufmgr = {
-   dri2_display_bind_wayland_display,
-   dri2_display_unbind_wayland_display,
-   egl_g3d_wl_drm_common_wl_buffer_get_resource,
-   egl_g3d_wl_drm_common_query_buffer
-};
-
-#endif /* HAVE_WAYLAND_BACKEND */
-
 struct native_display *
 x11_create_dri2_display(Display *dpy,
                         const struct native_event_handler *event_handler)
@@ -936,9 +908,6 @@ x11_create_dri2_display(Display *dpy,
    dri2dpy->base.copy_to_pixmap = native_display_copy_to_pixmap;
    dri2dpy->base.create_window_surface = dri2_display_create_window_surface;
    dri2dpy->base.create_pixmap_surface = dri2_display_create_pixmap_surface;
-#ifdef HAVE_WAYLAND_BACKEND
-   dri2dpy->base.wayland_bufmgr = &dri2_display_wayland_bufmgr;
-#endif
 
    return &dri2dpy->base;
 }
